@@ -8,7 +8,7 @@ set -euo pipefail
 # override default AWS region
 export AWS_REGION=eu-central-1
 
-LAYER_NAME=SentryPythonServerlessSDKLocalDev
+LAYER_NAME=SentryPythonServerlessSDK-local-dev
 VERSION="0"
 
 while [[ $VERSION != "1" ]]
