@@ -1,19 +1,27 @@
-from __future__ import absolute_import
+import functools
 
-from sentry_sdk.hub import Hub
+import sentry_sdk
 from sentry_sdk.tracing import SOURCE_FOR_STYLE
 from sentry_sdk.utils import (
     capture_internal_exceptions,
+    ensure_integration_enabled,
     event_from_exception,
+    parse_version,
     transaction_from_function,
 )
-from sentry_sdk.integrations import Integration, DidNotEnable
+from sentry_sdk.integrations import (
+    Integration,
+    DidNotEnable,
+    _DEFAULT_FAILED_REQUEST_STATUS_CODES,
+)
 from sentry_sdk.integrations.wsgi import SentryWsgiMiddleware
 from sentry_sdk.integrations._wsgi_common import RequestExtractor
 
-from sentry_sdk._types import MYPY
+from typing import TYPE_CHECKING
+
+if TYPE_CHECKING:
+    from collections.abc import Set
 
-if MYPY:
     from sentry_sdk.integrations.wsgi import _ScopedResponse
     from typing import Any
     from typing import Dict
@@ -26,9 +34,9 @@ if MYPY:
 try:
     from bottle import (
         Bottle,
+        HTTPResponse,
         Route,
         request as bottle_request,
-        HTTPResponse,
         __version__ as BOTTLE_VERSION,
     )
 except ImportError:
@@ -40,11 +48,17 @@ TRANSACTION_STYLE_VALUES = ("endpoint", "url")
 
 class BottleIntegration(Integration):
     identifier = "bottle"
+    origin = f"auto.http.{identifier}"
 
     transaction_style = ""
 
-    def __init__(self, transaction_style="endpoint"):
-        # type: (str) -> None
+    def __init__(
+        self,
+        transaction_style="endpoint",  # type: str
+        *,
+        failed_request_status_codes=_DEFAULT_FAILED_REQUEST_STATUS_CODES,  # type: Set[int]
+    ):
+        # type: (...) -> None
 
         if transaction_style not in TRANSACTION_STYLE_VALUES:
             raise ValueError(
@@ -52,93 +66,78 @@ class BottleIntegration(Integration):
                 % (transaction_style, TRANSACTION_STYLE_VALUES)
             )
         self.transaction_style = transaction_style
+        self.failed_request_status_codes = failed_request_status_codes
 
     @staticmethod
     def setup_once():
         # type: () -> None
+        version = parse_version(BOTTLE_VERSION)
 
-        try:
-            version = tuple(map(int, BOTTLE_VERSION.replace("-dev", "").split(".")))
-        except (TypeError, ValueError):
-            raise DidNotEnable("Unparsable Bottle version: {}".format(version))
+        if version is None:
+            raise DidNotEnable("Unparsable Bottle version: {}".format(BOTTLE_VERSION))
 
         if version < (0, 12):
             raise DidNotEnable("Bottle 0.12 or newer required.")
 
-        # monkey patch method Bottle.__call__
         old_app = Bottle.__call__
 
+        @ensure_integration_enabled(BottleIntegration, old_app)
         def sentry_patched_wsgi_app(self, environ, start_response):
             # type: (Any, Dict[str, str], Callable[..., Any]) -> _ScopedResponse
-
-            hub = Hub.current
-            integration = hub.get_integration(BottleIntegration)
-            if integration is None:
-                return old_app(self, environ, start_response)
-
-            return SentryWsgiMiddleware(lambda *a, **kw: old_app(self, *a, **kw))(
-                environ, start_response
+            middleware = SentryWsgiMiddleware(
+                lambda *a, **kw: old_app(self, *a, **kw),
+                span_origin=BottleIntegration.origin,
             )
 
+            return middleware(environ, start_response)
+
         Bottle.__call__ = sentry_patched_wsgi_app
 
-        # monkey patch method Bottle._handle
         old_handle = Bottle._handle
 
+        @functools.wraps(old_handle)
         def _patched_handle(self, environ):
             # type: (Bottle, Dict[str, Any]) -> Any
-            hub = Hub.current
-            integration = hub.get_integration(BottleIntegration)
+            integration = sentry_sdk.get_client().get_integration(BottleIntegration)
             if integration is None:
                 return old_handle(self, environ)
 
-            # create new scope
-            scope_manager = hub.push_scope()
+            scope = sentry_sdk.get_isolation_scope()
+            scope._name = "bottle"
+            scope.add_event_processor(
+                _make_request_event_processor(self, bottle_request, integration)
+            )
+            res = old_handle(self, environ)
 
-            with scope_manager:
-                app = self
-                with hub.configure_scope() as scope:
-                    scope._name = "bottle"
-                    scope.add_event_processor(
-                        _make_request_event_processor(app, bottle_request, integration)
-                    )
-                res = old_handle(self, environ)
-
-            # scope cleanup
             return res
 
         Bottle._handle = _patched_handle
 
-        # monkey patch method Route._make_callback
         old_make_callback = Route._make_callback
 
+        @functools.wraps(old_make_callback)
         def patched_make_callback(self, *args, **kwargs):
             # type: (Route, *object, **object) -> Any
-            hub = Hub.current
-            integration = hub.get_integration(BottleIntegration)
             prepared_callback = old_make_callback(self, *args, **kwargs)
+
+            integration = sentry_sdk.get_client().get_integration(BottleIntegration)
             if integration is None:
                 return prepared_callback
 
-            # If an integration is there, a client has to be there.
-            client = hub.client  # type: Any
-
             def wrapped_callback(*args, **kwargs):
                 # type: (*object, **object) -> Any
-
                 try:
                     res = prepared_callback(*args, **kwargs)
-                except HTTPResponse:
-                    raise
                 except Exception as exception:
-                    event, hint = event_from_exception(
-                        exception,
-                        client_options=client.options,
-                        mechanism={"type": "bottle", "handled": False},
-                    )
-                    hub.capture_event(event, hint=hint)
+                    _capture_exception(exception, handled=False)
                     raise exception
 
+                if (
+                    isinstance(res, HTTPResponse)
+                    and res.status_code in integration.failed_request_status_codes
+                ):
+                    _capture_exception(res, handled=True)
+
                 return res
 
             return wrapped_callback
@@ -199,7 +198,7 @@ def _make_request_event_processor(app, request, integration):
     # type: (Bottle, LocalRequest, BottleIntegration) -> EventProcessor
 
     def event_processor(event, hint):
-        # type: (Dict[str, Any], Dict[str, Any]) -> Dict[str, Any]
+        # type: (Event, dict[str, Any]) -> Event
         _set_transaction_name_and_source(event, integration.transaction_style, request)
 
         with capture_internal_exceptions():
@@ -208,3 +207,13 @@ def _make_request_event_processor(app, request, integration):
         return event
 
     return event_processor
+
+
+def _capture_exception(exception, handled):
+    # type: (BaseException, bool) -> None
+    event, hint = event_from_exception(
+        exception,
+        client_options=sentry_sdk.get_client().options,
+        mechanism={"type": "bottle", "handled": handled},
+    )
+    sentry_sdk.capture_event(event, hint=hint)
