@@ -1,19 +1,23 @@
 import asyncio
 import json
 from contextlib import suppress
+from unittest import mock
 
 import pytest
-from aiohttp import web
+from aiohttp import web, ClientSession
 from aiohttp.client import ServerDisconnectedError
 from aiohttp.web_request import Request
+from aiohttp.web_exceptions import (
+    HTTPInternalServerError,
+    HTTPNetworkAuthenticationRequired,
+    HTTPBadRequest,
+    HTTPNotFound,
+    HTTPUnavailableForLegalReasons,
+)
 
 from sentry_sdk import capture_message, start_transaction
 from sentry_sdk.integrations.aiohttp import AioHttpIntegration
-
-try:
-    from unittest import mock  # python 3.3 and above
-except ImportError:
-    import mock  # python < 3.3
+from tests.conftest import ApproxDict
 
 
 @pytest.mark.asyncio
@@ -51,7 +55,7 @@ async def test_basic(sentry_init, aiohttp_client, capture_events):
     assert request["url"] == "http://{host}/".format(host=host)
     assert request["headers"] == {
         "Accept": "*/*",
-        "Accept-Encoding": "gzip, deflate",
+        "Accept-Encoding": mock.ANY,
         "Host": host,
         "User-Agent": request["headers"]["User-Agent"],
         "baggage": mock.ANY,
@@ -256,12 +260,42 @@ async def test_transaction_style(
     assert event["transaction_info"] == {"source": expected_source}
 
 
+@pytest.mark.tests_internal_exceptions
+@pytest.mark.asyncio
+async def test_tracing_unparseable_url(sentry_init, aiohttp_client, capture_events):
+    sentry_init(integrations=[AioHttpIntegration()], traces_sample_rate=1.0)
+
+    async def hello(request):
+        return web.Response(text="hello")
+
+    app = web.Application()
+    app.router.add_get("/", hello)
+
+    events = capture_events()
+
+    client = await aiohttp_client(app)
+    with mock.patch(
+        "sentry_sdk.integrations.aiohttp.parse_url", side_effect=ValueError
+    ):
+        resp = await client.get("/")
+
+    assert resp.status == 200
+
+    (event,) = events
+
+    assert event["type"] == "transaction"
+    assert (
+        event["transaction"]
+        == "tests.integrations.aiohttp.test_aiohttp.test_tracing_unparseable_url.<locals>.hello"
+    )
+
+
 @pytest.mark.asyncio
 async def test_traces_sampler_gets_request_object_in_sampling_context(
     sentry_init,
     aiohttp_client,
-    DictionaryContaining,  # noqa:N803
-    ObjectDescribedBy,
+    DictionaryContaining,  # noqa: N803
+    ObjectDescribedBy,  # noqa: N803
 ):
     traces_sampler = mock.Mock()
     sentry_init(
@@ -377,13 +411,17 @@ async def test_trace_from_headers_if_performance_enabled(
     # The aiohttp_client is instrumented so will generate the sentry-trace header and add request.
     # Get the sentry-trace header from the request so we can later compare with transaction events.
     client = await aiohttp_client(app)
-    resp = await client.get("/")
+    with start_transaction():
+        # Headers are only added to the span if there is an active transaction
+        resp = await client.get("/")
+
     sentry_trace_header = resp.request_info.headers.get("sentry-trace")
     trace_id = sentry_trace_header.split("-")[0]
 
     assert resp.status == 500
 
-    msg_event, error_event, transaction_event = events
+    # Last item is the custom transaction event wrapping `client.get("/")`
+    msg_event, error_event, transaction_event, _ = events
 
     assert msg_event["contexts"]["trace"]
     assert "trace_id" in msg_event["contexts"]["trace"]
@@ -465,15 +503,17 @@ async def test_crumb_capture(
         crumb = event["breadcrumbs"]["values"][0]
         assert crumb["type"] == "http"
         assert crumb["category"] == "httplib"
-        assert crumb["data"] == {
-            "url": "http://127.0.0.1:{}/".format(raw_server.port),
-            "http.fragment": "",
-            "http.method": "GET",
-            "http.query": "",
-            "http.response.status_code": 200,
-            "reason": "OK",
-            "extra": "foo",
-        }
+        assert crumb["data"] == ApproxDict(
+            {
+                "url": "http://127.0.0.1:{}/".format(raw_server.port),
+                "http.fragment": "",
+                "http.method": "GET",
+                "http.query": "",
+                "http.response.status_code": 200,
+                "reason": "OK",
+                "extra": "foo",
+            }
+        )
 
 
 @pytest.mark.asyncio
@@ -534,3 +574,147 @@ async def test_outgoing_trace_headers_append_to_baggage(
             resp.request_info.headers["baggage"]
             == "custom=value,sentry-trace_id=0123456789012345678901234567890,sentry-environment=production,sentry-release=d08ebdb9309e1b004c6f52202de58a09c2268e42,sentry-transaction=/interactions/other-dogs/new-dog,sentry-sample_rate=1.0,sentry-sampled=true"
         )
+
+
+@pytest.mark.asyncio
+async def test_span_origin(
+    sentry_init,
+    aiohttp_client,
+    capture_events,
+):
+    sentry_init(
+        integrations=[AioHttpIntegration()],
+        traces_sample_rate=1.0,
+    )
+
+    async def hello(request):
+        async with ClientSession() as session:
+            async with session.get("http://example.com"):
+                return web.Response(text="hello")
+
+    app = web.Application()
+    app.router.add_get(r"/", hello)
+
+    events = capture_events()
+
+    client = await aiohttp_client(app)
+    await client.get("/")
+
+    (event,) = events
+    assert event["contexts"]["trace"]["origin"] == "auto.http.aiohttp"
+    assert event["spans"][0]["origin"] == "auto.http.aiohttp"
+
+
+@pytest.mark.parametrize(
+    ("integration_kwargs", "exception_to_raise", "should_capture"),
+    (
+        ({}, None, False),
+        ({}, HTTPBadRequest, False),
+        (
+            {},
+            HTTPUnavailableForLegalReasons(None),
+            False,
+        ),  # Highest 4xx status code (451)
+        ({}, HTTPInternalServerError, True),
+        ({}, HTTPNetworkAuthenticationRequired, True),  # Highest 5xx status code (511)
+        ({"failed_request_status_codes": set()}, HTTPInternalServerError, False),
+        (
+            {"failed_request_status_codes": set()},
+            HTTPNetworkAuthenticationRequired,
+            False,
+        ),
+        ({"failed_request_status_codes": {404, *range(500, 600)}}, HTTPNotFound, True),
+        (
+            {"failed_request_status_codes": {404, *range(500, 600)}},
+            HTTPInternalServerError,
+            True,
+        ),
+        (
+            {"failed_request_status_codes": {404, *range(500, 600)}},
+            HTTPBadRequest,
+            False,
+        ),
+    ),
+)
+@pytest.mark.asyncio
+async def test_failed_request_status_codes(
+    sentry_init,
+    aiohttp_client,
+    capture_events,
+    integration_kwargs,
+    exception_to_raise,
+    should_capture,
+):
+    sentry_init(integrations=[AioHttpIntegration(**integration_kwargs)])
+    events = capture_events()
+
+    async def handle(_):
+        if exception_to_raise is not None:
+            raise exception_to_raise
+        else:
+            return web.Response(status=200)
+
+    app = web.Application()
+    app.router.add_get("/", handle)
+
+    client = await aiohttp_client(app)
+    resp = await client.get("/")
+
+    expected_status = (
+        200 if exception_to_raise is None else exception_to_raise.status_code
+    )
+    assert resp.status == expected_status
+
+    if should_capture:
+        (event,) = events
+        assert event["exception"]["values"][0]["type"] == exception_to_raise.__name__
+    else:
+        assert not events
+
+
+@pytest.mark.asyncio
+async def test_failed_request_status_codes_with_returned_status(
+    sentry_init, aiohttp_client, capture_events
+):
+    """
+    Returning a web.Response with a failed_request_status_code should not be reported to Sentry.
+    """
+    sentry_init(integrations=[AioHttpIntegration(failed_request_status_codes={500})])
+    events = capture_events()
+
+    async def handle(_):
+        return web.Response(status=500)
+
+    app = web.Application()
+    app.router.add_get("/", handle)
+
+    client = await aiohttp_client(app)
+    resp = await client.get("/")
+
+    assert resp.status == 500
+    assert not events
+
+
+@pytest.mark.asyncio
+async def test_failed_request_status_codes_non_http_exception(
+    sentry_init, aiohttp_client, capture_events
+):
+    """
+    If an exception, which is not an instance of HTTPException, is raised, it should be captured, even if
+    failed_request_status_codes is empty.
+    """
+    sentry_init(integrations=[AioHttpIntegration(failed_request_status_codes=set())])
+    events = capture_events()
+
+    async def handle(_):
+        1 / 0
+
+    app = web.Application()
+    app.router.add_get("/", handle)
+
+    client = await aiohttp_client(app)
+    resp = await client.get("/")
+    assert resp.status == 500
+
+    (event,) = events
+    assert event["exception"]["values"][0]["type"] == "ZeroDivisionError"
