@@ -10,6 +10,7 @@ from clickhouse_driver import Client, connect
 
 from sentry_sdk import start_transaction, capture_message
 from sentry_sdk.integrations.clickhouse_driver import ClickhouseDriverIntegration
+from tests.conftest import ApproxDict
 
 EXPECT_PARAMS_IN_SELECT = True
 if clickhouse_driver.VERSION < (0, 2, 6):
@@ -102,6 +103,9 @@ def test_clickhouse_client_breadcrumbs(sentry_init, capture_events) -> None:
     if not EXPECT_PARAMS_IN_SELECT:
         expected_breadcrumbs[-1]["data"].pop("db.params", None)
 
+    for crumb in expected_breadcrumbs:
+        crumb["data"] = ApproxDict(crumb["data"])
+
     for crumb in event["breadcrumbs"]["values"]:
         crumb.pop("timestamp", None)
 
@@ -201,6 +205,9 @@ def test_clickhouse_client_breadcrumbs_with_pii(sentry_init, capture_events) ->
     if not EXPECT_PARAMS_IN_SELECT:
         expected_breadcrumbs[-1]["data"].pop("db.params", None)
 
+    for crumb in expected_breadcrumbs:
+        crumb["data"] = ApproxDict(crumb["data"])
+
     for crumb in event["breadcrumbs"]["values"]:
         crumb.pop("timestamp", None)
 
@@ -240,6 +247,7 @@ def test_clickhouse_client_spans(
     expected_spans = [
         {
             "op": "db",
+            "origin": "auto.db.clickhouse_driver",
             "description": "DROP TABLE IF EXISTS test",
             "data": {
                 "db.system": "clickhouse",
@@ -254,6 +262,7 @@ def test_clickhouse_client_spans(
         },
         {
             "op": "db",
+            "origin": "auto.db.clickhouse_driver",
             "description": "CREATE TABLE test (x Int32) ENGINE = Memory",
             "data": {
                 "db.system": "clickhouse",
@@ -268,6 +277,7 @@ def test_clickhouse_client_spans(
         },
         {
             "op": "db",
+            "origin": "auto.db.clickhouse_driver",
             "description": "INSERT INTO test (x) VALUES",
             "data": {
                 "db.system": "clickhouse",
@@ -282,6 +292,7 @@ def test_clickhouse_client_spans(
         },
         {
             "op": "db",
+            "origin": "auto.db.clickhouse_driver",
             "description": "INSERT INTO test (x) VALUES",
             "data": {
                 "db.system": "clickhouse",
@@ -296,6 +307,7 @@ def test_clickhouse_client_spans(
         },
         {
             "op": "db",
+            "origin": "auto.db.clickhouse_driver",
             "description": "SELECT sum(x) FROM test WHERE x > 150",
             "data": {
                 "db.system": "clickhouse",
@@ -313,6 +325,9 @@ def test_clickhouse_client_spans(
     if not EXPECT_PARAMS_IN_SELECT:
         expected_spans[-1]["data"].pop("db.params", None)
 
+    for span in expected_spans:
+        span["data"] = ApproxDict(span["data"])
+
     for span in event["spans"]:
         span.pop("span_id", None)
         span.pop("start_timestamp", None)
@@ -355,6 +370,7 @@ def test_clickhouse_client_spans_with_pii(
     expected_spans = [
         {
             "op": "db",
+            "origin": "auto.db.clickhouse_driver",
             "description": "DROP TABLE IF EXISTS test",
             "data": {
                 "db.system": "clickhouse",
@@ -370,6 +386,7 @@ def test_clickhouse_client_spans_with_pii(
         },
         {
             "op": "db",
+            "origin": "auto.db.clickhouse_driver",
             "description": "CREATE TABLE test (x Int32) ENGINE = Memory",
             "data": {
                 "db.system": "clickhouse",
@@ -385,6 +402,7 @@ def test_clickhouse_client_spans_with_pii(
         },
         {
             "op": "db",
+            "origin": "auto.db.clickhouse_driver",
             "description": "INSERT INTO test (x) VALUES",
             "data": {
                 "db.system": "clickhouse",
@@ -400,6 +418,7 @@ def test_clickhouse_client_spans_with_pii(
         },
         {
             "op": "db",
+            "origin": "auto.db.clickhouse_driver",
             "description": "INSERT INTO test (x) VALUES",
             "data": {
                 "db.system": "clickhouse",
@@ -415,6 +434,7 @@ def test_clickhouse_client_spans_with_pii(
         },
         {
             "op": "db",
+            "origin": "auto.db.clickhouse_driver",
             "description": "SELECT sum(x) FROM test WHERE x > 150",
             "data": {
                 "db.system": "clickhouse",
@@ -434,6 +454,9 @@ def test_clickhouse_client_spans_with_pii(
     if not EXPECT_PARAMS_IN_SELECT:
         expected_spans[-1]["data"].pop("db.params", None)
 
+    for span in expected_spans:
+        span["data"] = ApproxDict(span["data"])
+
     for span in event["spans"]:
         span.pop("span_id", None)
         span.pop("start_timestamp", None)
@@ -529,6 +552,9 @@ def test_clickhouse_dbapi_breadcrumbs(sentry_init, capture_events) -> None:
     if not EXPECT_PARAMS_IN_SELECT:
         expected_breadcrumbs[-1]["data"].pop("db.params", None)
 
+    for crumb in expected_breadcrumbs:
+        crumb["data"] = ApproxDict(crumb["data"])
+
     for crumb in event["breadcrumbs"]["values"]:
         crumb.pop("timestamp", None)
 
@@ -629,6 +655,9 @@ def test_clickhouse_dbapi_breadcrumbs_with_pii(sentry_init, capture_events) -> N
     if not EXPECT_PARAMS_IN_SELECT:
         expected_breadcrumbs[-1]["data"].pop("db.params", None)
 
+    for crumb in expected_breadcrumbs:
+        crumb["data"] = ApproxDict(crumb["data"])
+
     for crumb in event["breadcrumbs"]["values"]:
         crumb.pop("timestamp", None)
 
@@ -666,6 +695,7 @@ def test_clickhouse_dbapi_spans(sentry_init, capture_events, capture_envelopes)
     expected_spans = [
         {
             "op": "db",
+            "origin": "auto.db.clickhouse_driver",
             "description": "DROP TABLE IF EXISTS test",
             "data": {
                 "db.system": "clickhouse",
@@ -680,6 +710,7 @@ def test_clickhouse_dbapi_spans(sentry_init, capture_events, capture_envelopes)
         },
         {
             "op": "db",
+            "origin": "auto.db.clickhouse_driver",
             "description": "CREATE TABLE test (x Int32) ENGINE = Memory",
             "data": {
                 "db.system": "clickhouse",
@@ -694,6 +725,7 @@ def test_clickhouse_dbapi_spans(sentry_init, capture_events, capture_envelopes)
         },
         {
             "op": "db",
+            "origin": "auto.db.clickhouse_driver",
             "description": "INSERT INTO test (x) VALUES",
             "data": {
                 "db.system": "clickhouse",
@@ -708,6 +740,7 @@ def test_clickhouse_dbapi_spans(sentry_init, capture_events, capture_envelopes)
         },
         {
             "op": "db",
+            "origin": "auto.db.clickhouse_driver",
             "description": "INSERT INTO test (x) VALUES",
             "data": {
                 "db.system": "clickhouse",
@@ -722,6 +755,7 @@ def test_clickhouse_dbapi_spans(sentry_init, capture_events, capture_envelopes)
         },
         {
             "op": "db",
+            "origin": "auto.db.clickhouse_driver",
             "description": "SELECT sum(x) FROM test WHERE x > 150",
             "data": {
                 "db.system": "clickhouse",
@@ -739,6 +773,9 @@ def test_clickhouse_dbapi_spans(sentry_init, capture_events, capture_envelopes)
     if not EXPECT_PARAMS_IN_SELECT:
         expected_spans[-1]["data"].pop("db.params", None)
 
+    for span in expected_spans:
+        span["data"] = ApproxDict(span["data"])
+
     for span in event["spans"]:
         span.pop("span_id", None)
         span.pop("start_timestamp", None)
@@ -781,6 +818,7 @@ def test_clickhouse_dbapi_spans_with_pii(
     expected_spans = [
         {
             "op": "db",
+            "origin": "auto.db.clickhouse_driver",
             "description": "DROP TABLE IF EXISTS test",
             "data": {
                 "db.system": "clickhouse",
@@ -796,6 +834,7 @@ def test_clickhouse_dbapi_spans_with_pii(
         },
         {
             "op": "db",
+            "origin": "auto.db.clickhouse_driver",
             "description": "CREATE TABLE test (x Int32) ENGINE = Memory",
             "data": {
                 "db.system": "clickhouse",
@@ -811,6 +850,7 @@ def test_clickhouse_dbapi_spans_with_pii(
         },
         {
             "op": "db",
+            "origin": "auto.db.clickhouse_driver",
             "description": "INSERT INTO test (x) VALUES",
             "data": {
                 "db.system": "clickhouse",
@@ -826,6 +866,7 @@ def test_clickhouse_dbapi_spans_with_pii(
         },
         {
             "op": "db",
+            "origin": "auto.db.clickhouse_driver",
             "description": "INSERT INTO test (x) VALUES",
             "data": {
                 "db.system": "clickhouse",
@@ -841,6 +882,7 @@ def test_clickhouse_dbapi_spans_with_pii(
         },
         {
             "op": "db",
+            "origin": "auto.db.clickhouse_driver",
             "description": "SELECT sum(x) FROM test WHERE x > 150",
             "data": {
                 "db.system": "clickhouse",
@@ -860,9 +902,31 @@ def test_clickhouse_dbapi_spans_with_pii(
     if not EXPECT_PARAMS_IN_SELECT:
         expected_spans[-1]["data"].pop("db.params", None)
 
+    for span in expected_spans:
+        span["data"] = ApproxDict(span["data"])
+
     for span in event["spans"]:
         span.pop("span_id", None)
         span.pop("start_timestamp", None)
         span.pop("timestamp", None)
 
     assert event["spans"] == expected_spans
+
+
+def test_span_origin(sentry_init, capture_events, capture_envelopes) -> None:
+    sentry_init(
+        integrations=[ClickhouseDriverIntegration()],
+        traces_sample_rate=1.0,
+    )
+
+    events = capture_events()
+
+    with start_transaction(name="test_clickhouse_transaction"):
+        conn = connect("clickhouse://localhost")
+        cursor = conn.cursor()
+        cursor.execute("SELECT 1")
+
+    (event,) = events
+
+    assert event["contexts"]["trace"]["origin"] == "manual"
+    assert event["spans"][0]["origin"] == "auto.db.clickhouse_driver"
