@@ -1,10 +1,11 @@
+import asyncio
 import json
 import threading
 
-from django import VERSION
 from django.contrib.auth import login
 from django.contrib.auth.models import User
 from django.core.exceptions import PermissionDenied
+from django.dispatch import Signal
 from django.http import HttpResponse, HttpResponseNotFound, HttpResponseServerError
 from django.shortcuts import render
 from django.template import Context, Template
@@ -14,6 +15,12 @@ from django.views.decorators.cache import cache_page
 from django.views.decorators.csrf import csrf_exempt
 from django.views.generic import ListView
 
+
+from tests.integrations.django.myapp.signals import (
+    myapp_custom_signal,
+    myapp_custom_signal_silenced,
+)
+
 try:
     from rest_framework.decorators import api_view
     from rest_framework.response import Response
@@ -84,14 +91,14 @@ def view_with_cached_template_fragment(request):
 # interesting property of this one is that csrf_exempt, as a class attribute,
 # is not in __dict__, so regular use of functools.wraps will not forward the
 # attribute.
-class SentryClassBasedView(object):
+class SentryClassBasedView:
     csrf_exempt = True
 
     def __call__(self, request):
         return HttpResponse("ok")
 
 
-class SentryClassBasedViewWithCsrf(object):
+class SentryClassBasedViewWithCsrf:
     def __call__(self, request):
         return HttpResponse("ok")
 
@@ -108,6 +115,18 @@ def message(request):
     return HttpResponse("ok")
 
 
+@csrf_exempt
+def nomessage(request):
+    return HttpResponse("ok")
+
+
+@csrf_exempt
+def view_with_signal(request):
+    custom_signal = Signal()
+    custom_signal.send(sender="hello")
+    return HttpResponse("ok")
+
+
 @csrf_exempt
 def mylogin(request):
     user = User.objects.create_user("john", "lennon@thebeatles.com", "johnpassword")
@@ -118,7 +137,7 @@ def mylogin(request):
 
 @csrf_exempt
 def handler500(request):
-    return HttpResponseServerError("Sentry error: %s" % sentry_sdk.last_event_id())
+    return HttpResponseServerError("Sentry error.")
 
 
 class ClassBasedView(ListView):
@@ -126,7 +145,7 @@ class ClassBasedView(ListView):
 
     @method_decorator(csrf_exempt)
     def dispatch(self, request, *args, **kwargs):
-        return super(ClassBasedView, self).dispatch(request, *args, **kwargs)
+        return super().dispatch(request, *args, **kwargs)
 
     def head(self, *args, **kwargs):
         sentry_sdk.capture_message("hi")
@@ -177,10 +196,15 @@ def template_test2(request, *args, **kwargs):
 
 @csrf_exempt
 def template_test3(request, *args, **kwargs):
-    from sentry_sdk import Hub
+    traceparent = sentry_sdk.get_current_scope().get_traceparent()
+    if traceparent is None:
+        traceparent = sentry_sdk.get_isolation_scope().get_traceparent()
 
-    hub = Hub.current
-    capture_message(hub.get_traceparent() + "\n" + hub.get_baggage())
+    baggage = sentry_sdk.get_current_scope().get_baggage()
+    if baggage is None:
+        baggage = sentry_sdk.get_isolation_scope().get_baggage()
+
+    capture_message(traceparent + "\n" + baggage.serialize())
     return render(request, "trace_meta.html", {})
 
 
@@ -218,38 +242,40 @@ def thread_ids_sync(*args, **kwargs):
     return HttpResponse(response)
 
 
-if VERSION >= (3, 1):
-    # Use exec to produce valid Python 2
-    exec(
-        """async def async_message(request):
+async def async_message(request):
     sentry_sdk.capture_message("hi")
-    return HttpResponse("ok")"""
-    )
+    return HttpResponse("ok")
 
-    exec(
-        """async def my_async_view(request):
-    import asyncio
+
+async def my_async_view(request):
     await asyncio.sleep(1)
-    return HttpResponse('Hello World')"""
-    )
+    return HttpResponse("Hello World")
 
-    exec(
-        """async def thread_ids_async(request):
-    response = json.dumps({
-        "main": threading.main_thread().ident,
-        "active": threading.current_thread().ident,
-    })
-    return HttpResponse(response)"""
-    )
 
-    exec(
-        """async def post_echo_async(request):
+async def simple_async_view(request):
+    return HttpResponse("Simple Hello World")
+
+
+async def thread_ids_async(request):
+    response = json.dumps(
+        {
+            "main": threading.main_thread().ident,
+            "active": threading.current_thread().ident,
+        }
+    )
+    return HttpResponse(response)
+
+
+async def post_echo_async(request):
     sentry_sdk.capture_message("hi")
     return HttpResponse(request.body)
-post_echo_async.csrf_exempt = True"""
-    )
-else:
-    async_message = None
-    my_async_view = None
-    thread_ids_async = None
-    post_echo_async = None
+
+
+post_echo_async.csrf_exempt = True
+
+
+@csrf_exempt
+def send_myapp_custom_signal(request):
+    myapp_custom_signal.send(sender="hello")
+    myapp_custom_signal_silenced.send(sender="hello")
+    return HttpResponse("ok")
