@@ -9,6 +9,6 @@ def test_basic(sentry_init, capture_events):
 
     sentry_sdk.capture_exception(ValueError())
 
-    event, = events
+    (event,) = events
     assert "sentry-sdk" in event["modules"]
     assert "pytest" in event["modules"]
