@@ -2,6 +2,7 @@ import socket
 
 from sentry_sdk import start_transaction
 from sentry_sdk.integrations.socket import SocketIntegration
+from tests.conftest import ApproxDict
 
 
 def test_getaddrinfo_trace(sentry_init, capture_events):
@@ -16,10 +17,12 @@ def test_getaddrinfo_trace(sentry_init, capture_events):
 
     assert span["op"] == "socket.dns"
     assert span["description"] == "example.com:443"
-    assert span["data"] == {
-        "host": "example.com",
-        "port": 443,
-    }
+    assert span["data"] == ApproxDict(
+        {
+            "host": "example.com",
+            "port": 443,
+        }
+    )
 
 
 def test_create_connection_trace(sentry_init, capture_events):
@@ -37,15 +40,40 @@ def test_create_connection_trace(sentry_init, capture_events):
 
     assert connect_span["op"] == "socket.connection"
     assert connect_span["description"] == "example.com:443"
-    assert connect_span["data"] == {
-        "address": ["example.com", 443],
-        "timeout": timeout,
-        "source_address": None,
-    }
+    assert connect_span["data"] == ApproxDict(
+        {
+            "address": ["example.com", 443],
+            "timeout": timeout,
+            "source_address": None,
+        }
+    )
 
     assert dns_span["op"] == "socket.dns"
     assert dns_span["description"] == "example.com:443"
-    assert dns_span["data"] == {
-        "host": "example.com",
-        "port": 443,
-    }
+    assert dns_span["data"] == ApproxDict(
+        {
+            "host": "example.com",
+            "port": 443,
+        }
+    )
+
+
+def test_span_origin(sentry_init, capture_events):
+    sentry_init(
+        integrations=[SocketIntegration()],
+        traces_sample_rate=1.0,
+    )
+    events = capture_events()
+
+    with start_transaction(name="foo"):
+        socket.create_connection(("example.com", 443), 1, None)
+
+    (event,) = events
+
+    assert event["contexts"]["trace"]["origin"] == "manual"
+
+    assert event["spans"][0]["op"] == "socket.connection"
+    assert event["spans"][0]["origin"] == "auto.socket.socket"
+
+    assert event["spans"][1]["op"] == "socket.dns"
+    assert event["spans"][1]["origin"] == "auto.socket.socket"
