@@ -1,5 +1,6 @@
 import random
 from http.client import HTTPConnection, HTTPSConnection
+from socket import SocketIO
 from urllib.request import urlopen
 from unittest import mock
 
@@ -342,3 +343,35 @@ def test_span_origin(sentry_init, capture_events):
 
     assert event["spans"][0]["op"] == "http.client"
     assert event["spans"][0]["origin"] == "auto.http.stdlib.httplib"
+
+
+def test_http_timeout(monkeypatch, sentry_init, capture_envelopes):
+    mock_readinto = mock.Mock(side_effect=TimeoutError)
+    monkeypatch.setattr(SocketIO, "readinto", mock_readinto)
+
+    sentry_init(traces_sample_rate=1.0)
+
+    envelopes = capture_envelopes()
+
+    with start_transaction(op="op", name="name"):
+        try:
+            conn = HTTPSConnection("www.squirrelchasers.com")
+            conn.request("GET", "/top-chasers")
+            conn.getresponse()
+        except Exception:
+            pass
+
+    items = [
+        item
+        for envelope in envelopes
+        for item in envelope.items
+        if item.type == "transaction"
+    ]
+    assert len(items) == 1
+
+    transaction = items[0].payload.json
+    assert len(transaction["spans"]) == 1
+
+    span = transaction["spans"][0]
+    assert span["op"] == "http.client"
+    assert span["description"] == "GET https://www.squirrelchasers.com/top-chasers"
